#include <ccblkfn.h>
#include "adi_initialize.h"

/* CCLK frequency (HZ) */
#define BFIN_CORE_CLOCK			(500000000)
/* SCLK0 frequency (HZ) */
#define BFIN_SCLK0_CLOCK		(125000000)
/* SCLK1 frequency (HZ) */
#define BFIN_SCLK1_CLOCK		(125000000)

#define CONFIG_PLL_MULT			(20)
#define CONFIG_PLL_CLKIN_HALF	(0)
#define CONFIG_PLL_CCLK_DIV		(1)
#define CONFIG_PLL_SCLK_DIV		(2)
#define CONFIG_PLL_SCLK0_DIV	(2)
#define CONFIG_PLL_SCLK1_DIV	(2)
#define CONFIG_PLL_DCLK_DIV		(2)
#define CONFIG_PLL_OCLK_DIV		(16)

#define REG_CGU0_DIV_VAL \
	((CONFIG_PLL_CCLK_DIV  << BITP_CGU_DIV_CSEL)   | \
	 (CONFIG_PLL_SCLK0_DIV << BITP_CGU_DIV_S0SEL)  | \
	 (CONFIG_PLL_SCLK_DIV  << BITP_CGU_DIV_SYSSEL) | \
	 (CONFIG_PLL_SCLK1_DIV << BITP_CGU_DIV_S1SEL)  | \
	 (CONFIG_PLL_DCLK_DIV  << BITP_CGU_DIV_DSEL)   | \
	 (CONFIG_PLL_OCLK_DIV  << BITP_CGU_DIV_OSEL))

#define REG_CGU0_CTL_VAL \
	((CONFIG_PLL_MULT << BITP_CGU_CTL_MSEL) | (CONFIG_PLL_CLKIN_HALF << BITP_CGU_CTL_DF))

void bfin_cgu_pll_set(void)
{
	if((*pREG_CGU0_CTL & (BITM_CGU_CTL_MSEL | BITM_CGU_CTL_DF)) != REG_CGU0_CTL_VAL)
	{
		*pREG_CGU0_DIV = REG_CGU0_DIV_VAL;
		*pREG_CGU0_CTL = REG_CGU0_CTL_VAL;
		while ((*pREG_CGU0_STAT & (BITM_CGU_STAT_CLKSALGN | BITM_CGU_STAT_PLLBP)) ||
			  !(*pREG_CGU0_STAT & BITM_CGU_STAT_PLOCK))
		{
			continue;
		}
	}

	*pREG_CGU0_DIV = REG_CGU0_DIV_VAL | BITM_CGU_DIV_UPDT;
	while(*pREG_CGU0_STAT & BITM_CGU_STAT_CLKSALGN)
		continue;

	if((*pREG_DMC0_STAT & BITM_DMC_STAT_MEMINITDONE) != 0u)
	{
		*pREG_DMC0_CTL &= ~BITP_DMC_CTL_SRREQ;
		ssync();
		while((*pREG_DMC0_STAT & BITM_DMC_STAT_SRACK) != 0u)
			continue;
	}
}

void s_pwm_init(void)
{
	// PWM1_AHsPWMo
	*pREG_PORTG_FER_SET = BITM_PORT_FER_SET_PX3;
	*pREG_PORTG_MUX |= (1U << BITP_PORT_MUX_MUX3);

	*pREG_PWM1_CTL = 0U;
	*pREG_PWM1_CHANCFG =
		ENUM_PWM_CHANCFG_REFTMRA_0;
	*pREG_PWM1_TM0 = 6250;
	*pREG_PWM1_ACTL =
		ENUM_PWM_SYM_HI |
		ENUM_PWM_LO_EN;
	*pREG_PWM1_AH0 = 6250/4;
	*pREG_PWM1_CTL =
		ENUM_PWM_CTL_INTSYNC_0 |
		ENUM_PWM_CTL_INTSYNC |
		ENUM_PWM_CTL_EXTSYNC_SYNC |
		ENUM_PWM_CTL_EMURUN_EN |
		ENUM_PWM_CTL_PWM_EN;
	ssync();
}

int main(void)
{
	adi_initComponents();
	bfin_cgu_pll_set();

	s_pwm_init();

	while(1)
	{
		ssync();
	}
	return 0;
}
